@@ -15,7 +15,9 @@
 
 """Main file for the pylint_secure_coding_standard plugin."""
 
+import operator
 import platform
+import stat
 
 import astroid
 from pylint.checkers import BaseChecker
@@ -32,6 +34,68 @@ def _is_posix():
     return platform.system() in ('Linux', 'Darwin')
 
 
+_is_unix = _is_posix
+
+# ==============================================================================
+
+
+def _read_octal_mode_option(name, value, default):
+    """
+    Read an integer or list of integer configuration option.
+
+    Args:
+        name (str): Name of option
+        value (str): Value of option from the configuration file or on the CLI. Its value can be any of:
+            - 'yes', 'y', 'true' (case-insensitive)
+                The maximum mode value is then set to self.DEFAULT_MAX_MODE
+            - a single octal or decimal integer
+                The maximum mode value is then set to that integer value
+            - a comma-separated list of integers (octal or decimal)
+                The allowed mode values are then those found in the list
+            - anything else will count as a falseful value
+        default (int,list): Default value for option if set to one of
+            ('y', 'yes', 'true') in the configuration file or on the CLI
+
+    Returns:
+        A single integer or a (possibly empty) list of integers
+
+    Raises:
+        ValueError: if the value of the option is not valid
+    """
+
+    def _str_to_int(arg):
+        try:
+            return int(arg, 8)
+        except ValueError:
+            return int(arg)
+
+    value = value.lower()
+    modes = [mode.strip() for mode in value.split(',')]
+
+    if len(modes) > 1:
+        # Lists of allowed modes
+        try:
+            allowed_modes = [_str_to_int(mode) for mode in modes if mode]
+        except ValueError as error:
+            raise ValueError(f'Unable to convert {modes} elements to integers!') from error
+        else:
+            if not allowed_modes:
+                raise ValueError(f'Calculated empty value for `{name}`!')
+            return allowed_modes
+    elif modes and modes[0]:
+        # Single values (ie. max allowed value for mode)
+        try:
+            return _str_to_int(value)
+        except ValueError as error:
+            if value in ('y', 'yes', 'true'):
+                return default
+            if value in ('n', 'no', 'false'):
+                return None
+            raise ValueError(f'Invalid value for `{name}`: {value}!') from error
+    else:
+        raise ValueError(f'Invalid value for `{name}`: {value}!')
+
+
 # ==============================================================================
 
 
@@ -91,26 +155,21 @@ def _is_builtin_open_for_writing(node):
     return False
 
 
-def _is_os_open_allowed_mode(node, allowed_modes):
+def _get_mode_arg(node, args_idx):
     mode = None
-    flags = None  # pylint: disable=unused-variable
-    if len(node.args) > 1 and isinstance(node.args[1], (astroid.Attribute, astroid.BinOp)):
-        # Cover:
-        #  * os.open(xxx, os.O_WRONLY)
-        #  * os.open(xxx, os.O_WRONLY | os.O_CREATE)
-        #  * os.open(xxx, os.O_WRONLY | os.O_CREATE | os.O_FSYNC)
-        flags = node.args[1]
-    if len(node.args) > 2 and isinstance(node.args[2], astroid.Const):
-        mode = node.args[2].value
+    if len(node.args) > args_idx and isinstance(node.args[args_idx], astroid.Const):
+        mode = node.args[args_idx].value
     elif node.keywords:
         for keyword in node.keywords:
-            if keyword.arg == 'flags' and isinstance(keyword.value, (astroid.Attribute, astroid.BinOp)):
-                flags = keyword.value  # pylint: disable=unused-variable # noqa: F841
             if keyword.arg == 'mode' and isinstance(keyword.value, astroid.Const):
                 mode = keyword.value.value
                 break
+    return mode
+
+
+def _is_allowed_mode(node, allowed_modes, args_idx):
+    mode = _get_mode_arg(node, args_idx=args_idx)
     if mode is not None:
-        # TODO: condition check on the flags value if present (ie. ignore if read-only)
         return mode in allowed_modes
 
     # NB: default to True in all other cases
@@ -224,12 +283,98 @@ def _is_yaml_unsafe_call(node):
 
 # ==============================================================================
 
+_unop = {'-': operator.neg, 'not': operator.not_, '~': operator.inv}
+_binop = {
+    '+': operator.add,
+    '-': operator.sub,
+    '*': operator.mul,
+    '/': operator.truediv,
+    '//': operator.floordiv,
+    '%': operator.mod,
+    '^': operator.xor,
+    '|': operator.or_,
+    '&': operator.and_,
+}
+_chmod_known_mode_values = (
+    'S_ISUID',
+    'S_ISGID',
+    'S_ENFMT',
+    'S_ISVTX',
+    'S_IREAD',
+    'S_IWRITE',
+    'S_IEXEC',
+    'S_IRWXU',
+    'S_IRUSR',
+    'S_IWUSR',
+    'S_IXUSR',
+    'S_IRWXG',
+    'S_IRGRP',
+    'S_IWGRP',
+    'S_IXGRP',
+    'S_IRWXO',
+    'S_IROTH',
+    'S_IWOTH',
+    'S_IXOTH',
+)
 
-class SecureCodingStandardChecker(BaseChecker):
+
+def _chmod_get_mode(node):
+    """
+    Extract the mode constant of a node.
+
+    Args:
+        node (astroid.node_classes.NodeNG): an AST node
+
+    Raises:
+        ValueError: if a node is encountered that cannot be processed
+    """
+    if isinstance(node, astroid.Name) and node.name in _chmod_known_mode_values:
+        return getattr(stat, node.name)
+    if (
+        isinstance(node, astroid.Attribute)
+        and isinstance(node.expr, astroid.Name)
+        and node.attrname in _chmod_known_mode_values
+        and node.expr.name == 'stat'
+    ):
+        return getattr(stat, node.attrname)
+    if isinstance(node, astroid.UnaryOp):
+        return _unop[node.op](_chmod_get_mode(node.operand))
+    if isinstance(node, astroid.BinOp):
+        return _binop[node.op](_chmod_get_mode(node.left), _chmod_get_mode(node.right))
+
+    raise ValueError(f'Do not know how to process node: {node.repr_tree()}')
+
+
+def _chmod_has_wx_for_go(node):
+    if platform.system() == 'Windows':
+        # On Windows, only stat.S_IREAD and stat.S_IWRITE can be used, all other bits are ignored
+        return False
+
+    try:
+        modes = None
+        if len(node.args) > 1:
+            modes = _chmod_get_mode(node.args[1])
+        elif node.keywords:
+            for keyword in node.keywords:
+                if keyword.arg == 'mode':
+                    modes = _chmod_get_mode(keyword.value)
+                    break
+    except ValueError:
+        return False
+    else:
+        if modes is None:
+            # NB: this would be from invalid code such as `os.chmod("file.txt")`
+            raise RuntimeError('Unable to extract `mode` argument from function call!')
+        return bool(modes & (stat.S_IWGRP | stat.S_IXGRP | stat.S_IWOTH | stat.S_IXOTH))
+
+
+# ==============================================================================
+
+
+class SecureCodingStandardChecker(BaseChecker):  # pylint: disable=too-many-instance-attributes
     """Plugin class."""
 
     DEFAULT_MAX_MODE = 0o755
-    W8012_DISPLAY_MSG = 'Avoid using `os.open` with unsafe permissions permissions'
 
     __implements__ = (IAstroidChecker,)
 
@@ -238,17 +383,44 @@ class SecureCodingStandardChecker(BaseChecker):
         (
             'os-open-mode',
             {
-                'default': False,
+                'default': '0',
                 'type': 'string',
                 'metavar': '<os-open-mode>',
                 'help': 'Integer or comma-separated list of integers (octal or decimal) of allowed modes. If set to a '
                 'truthful value (ie. >0 or non-empty list), this checker will prefer `os.open` over the builtin `open`',
             },
         ),
+        (
+            'os-mkdir-mode',
+            {
+                'default': '0',
+                'type': 'string',
+                'metavar': '<os-mkdir-mode>',
+                'help': 'Integer or comma-separated list of integers (octal or decimal) of allowed modes.',
+            },
+        ),
+        (
+            'os-mkfifo-mode',
+            {
+                'default': '0',
+                'type': 'string',
+                'metavar': '<os-mkfifo-mode>',
+                'help': 'Integer or comma-separated list of integers (octal or decimal) of allowed modes.',
+            },
+        ),
+        (
+            'os-mknod-mode',
+            {
+                'default': '0',
+                'type': 'string',
+                'metavar': '<os-mknod-mode>',
+                'help': 'Integer or comma-separated list of integers (octal or decimal) of allowed modes.',
+            },
+        ),
     )
     priority = -1
 
-    msg = {
+    msgs = {
         'R8000': (
             'Use `os.path.realpath()` instead of `os.path.abspath()` and `os.path.relpath()`',
             'replace-os-relpath-abspath',
@@ -312,7 +484,7 @@ class SecureCodingStandardChecker(BaseChecker):
             'Use of `shlex.quote()` should be avoided on non-POSIX platforms (such as Windows)',
         ),
         'W8012': (
-            W8012_DISPLAY_MSG,
+            'Avoid using `os.open` with unsafe permissions (should be %s)',
             'os-open-unsafe-permissions',
             'Avoid using `os.open` with unsafe file permissions (by default 0 <= mode <= 0o755)',
         ),
@@ -331,13 +503,39 @@ class SecureCodingStandardChecker(BaseChecker):
             'avoid-shelve-open',
             'Use of `shelve.open()` should be avoided in favour of safer file formats',
         ),
+        'W8016': (
+            'Avoid using `os.mkdir` and `os.makedirs` with unsafe permissions (should be %s)',
+            'os-mkdir-unsafe-permissions',
+            'Avoid using `os.mkdir` and `os.makedirs` with unsafe file permissions (by default 0 <= mode <= 0o755)',
+        ),
+        'W8017': (
+            'Avoid using `os.mkfifo` with unsafe permissions (should be %s)',
+            'os-mkfifo-unsafe-permissions',
+            'Avoid using `os.mkfifo` with unsafe file permissions (by default 0 <= mode <= 0o755)',
+        ),
+        'W8018': (
+            'Avoid using `os.mknod` with unsafe permissions (should be %s)',
+            'os-mknod-unsafe-permissions',
+            'Avoid using `os.mknod` with unsafe file permissions (by default 0 <= mode <= 0o755)',
+        ),
+        'W8019': (
+            'Avoid using `os.chmod` with unsafe permissions (W ^ X for group and others)',
+            'os-chmod-unsafe-permissions',
+            'Avoid using `os.chmod` with unsafe file permissions (W ^ X for group and others)',
+        ),
     }
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, linter):
         """Initialize a SecureCodingStandardChecker object."""
-        super().__init__(*args, **kwargs)
-        self._prefer_os_open = False
+        super().__init__(linter)
+        self._os_open_msg_arg = ''
         self._os_open_modes_allowed = []
+        self._os_mkdir_msg_arg = ''
+        self._os_mkdir_modes_allowed = []
+        self._os_mkfifo_msg_arg = ''
+        self._os_mkfifo_modes_allowed = []
+        self._os_mknod_msg_arg = ''
+        self._os_mknod_modes_allowed = []
 
     def visit_call(self, node):  # pylint: disable=too-many-branches
         """Visitor method called for astroid.Call nodes."""
@@ -357,7 +555,7 @@ class SecureCodingStandardChecker(BaseChecker):
             self.add_message('avoid-shell-true', node=node)
         elif _is_function_call(node, module='os', function='popen'):
             self.add_message('avoid-os-popen', node=node)
-        elif _is_builtin_open_for_writing(node) and self._prefer_os_open:
+        elif _is_builtin_open_for_writing(node) and self._os_open_modes_allowed:
             self.add_message('replace-builtin-open', node=node)
         elif isinstance(node.func, astroid.Name) and (node.func.name in ('eval', 'exec')):
             self.add_message('avoid-eval-exec', node=node)
@@ -365,16 +563,37 @@ class SecureCodingStandardChecker(BaseChecker):
             self.add_message('avoid-shlex-quote-on-non-posix', node=node)
         elif (
             _is_function_call(node, module='os', function='open')
-            and self._prefer_os_open
-            and not _is_os_open_allowed_mode(node, self._os_open_modes_allowed)
+            and self._os_open_modes_allowed
+            and not _is_allowed_mode(node, self._os_open_modes_allowed, args_idx=2)
         ):
-            self.add_message('os-open-unsafe-permissions', node=node)
+            self.add_message('os-open-unsafe-permissions', node=node, args=(self._os_open_msg_arg,))
         elif _is_function_call(node, module='pickle', function=('load', 'loads')):
             self.add_message('avoid-pickle-load', node=node)
         elif _is_function_call(node, module='marshal', function=('load', 'loads')):
             self.add_message('avoid-marshal-load', node=node)
         elif _is_function_call(node, module='shelve', function='open'):
             self.add_message('avoid-shelve-open', node=node)
+        elif _is_function_call(node, module='os', function='chmod') and _chmod_has_wx_for_go(node):
+            self.add_message('os-chmod-unsafe-permissions', node=node)
+        elif _is_unix():
+            if (
+                _is_function_call(node, module='os', function=('mkdir', 'makedirs'))
+                and self._os_mkdir_modes_allowed
+                and not _is_allowed_mode(node, self._os_mkdir_modes_allowed, args_idx=1)
+            ):
+                self.add_message('os-mkdir-unsafe-permissions', node=node, args=(self._os_mkdir_msg_arg,))
+            elif (
+                _is_function_call(node, module='os', function='mkfifo')
+                and self._os_mkfifo_modes_allowed
+                and not _is_allowed_mode(node, self._os_mkfifo_modes_allowed, args_idx=1)
+            ):
+                self.add_message('os-mkfifo-unsafe-permissions', node=node, args=(self._os_mkfifo_msg_arg,))
+            elif (
+                _is_function_call(node, module='os', function='mknod')
+                and self._os_mknod_modes_allowed
+                and not _is_allowed_mode(node, self._os_mknod_modes_allowed, args_idx=1)
+            ):
+                self.add_message('os-mknod-unsafe-permissions', node=node, args=(self._os_mknod_msg_arg,))
 
     def visit_import(self, node):
         """Visitor method called for astroid.Import nodes."""
@@ -415,11 +634,11 @@ class SecureCodingStandardChecker(BaseChecker):
         """Visitor method called for astroid.With nodes."""
         for item in node.items:
             if item and isinstance(item[0], astroid.Call):
-                if self._prefer_os_open:
+                if self._os_open_modes_allowed:
                     if _is_builtin_open_for_writing(item[0]):
                         self.add_message('replace-builtin-open', node=node)
-                    elif _is_function_call(item[0], module='os', function='open') and not _is_os_open_allowed_mode(
-                        item[0], self._os_open_modes_allowed
+                    elif _is_function_call(item[0], module='os', function='open') and not _is_allowed_mode(
+                        item[0], self._os_open_modes_allowed, args_idx=2
                     ):
                         self.add_message('os-open-unsafe-permissions', node=node)
                 elif _is_function_call(item[0], module='shelve', function='open'):
@@ -429,65 +648,65 @@ class SecureCodingStandardChecker(BaseChecker):
         """Visitor method called for astroid.Assert nodes."""
         self.add_message('avoid-assert', node=node)
 
-    def set_os_open_mode(self, arg):
+    def _set_mode_option(self, config_name, name, value):
+        modes = _read_octal_mode_option(config_name, value, self.DEFAULT_MAX_MODE)
+
+        if isinstance(modes, int) and modes > 0:
+            setattr(self, f'_os_{name}_modes_allowed', list(range(0, modes + 1)))
+            setattr(self, f'_os_{name}_msg_arg', f'0 < mode < {oct(modes)}')
+        elif modes:
+            setattr(self, f'_os_{name}_modes_allowed', modes)
+            setattr(self, f'_os_{name}_msg_arg', f'mode in {[oct(mode) for mode in modes]}')
+        else:
+            getattr(self, f'_os_{name}_modes_allowed').clear()
+
+    def set_os_open_allowed_modes(self, value):
         """
-        Control whether we prefer `os.open` over the builtin `open`.
+        Set the allowed modes for `os.open`.
+
+        Note:
+            This option has no effect on non-POSIX platforms.
 
         Args:
-            arg (str): String with with mode value. Can be either of:
-                - 'yes', 'y', 'true' (case-insensitive)
-                    The maximum mode value is then set to self.DEFAULT_MAX_MODE
-                - a single octal or decimal integer
-                    The maximum mode value is then set to that integer value
-                - a comma-separated list of integers (octal or decimal)
-                    The allowed mode values are then those found in the list
-                - anything else will disable the feature
+            value (str): Option value
         """
+        self._set_mode_option('os_open_mode', 'open', value)
 
-        def _str_to_int(arg):
-            try:
-                return int(arg, 8)
-            except ValueError:
-                return int(arg)
+    def set_os_mkdir_allowed_modes(self, value):
+        """
+        Set the allowed modes for `os.mkdir` and `os.makedirs`.
 
-        def _update_display_msg(suffix=''):
-            self.msg['W8012'] = (self.W8012_DISPLAY_MSG + suffix, self.msg['W8012'][1], self.msg['W8012'][2])
+        Note:
+            This option has no effect on non-UNIX platforms.
 
-        arg = arg.lower()
-        modes = [mode.strip() for mode in arg.split(',')]
+        Args:
+            value (str): Option value
+        """
+        self._set_mode_option('os_mkdir_mode', 'mkdir', value)
 
-        if len(modes) > 1:
-            # Lists of allowed modes
-            try:
-                self._os_open_modes_allowed = [_str_to_int(mode) for mode in modes if mode]
-                if not self._os_open_modes_allowed:
-                    raise ValueError('Calculated empty value for `os_open_mode`!')
-                self._prefer_os_open = True
-                _update_display_msg(suffix=f' (mode in {modes})')
-            except ValueError as error:
-                raise ValueError(f'Unable to convert {modes} elements to integers!') from error
-        elif modes and modes[0]:
-            # Single values (ie. max allowed value for mode)
-            try:
-                val = _str_to_int(arg)
-                self._prefer_os_open = val > 0
-                if self._prefer_os_open:
-                    self._os_open_modes_allowed = list(range(0, val + 1))
-                    _update_display_msg(suffix=f' (mode <= {arg})')
-                else:
-                    self._os_open_modes_allowed.clear()
-            except ValueError as error:
-                if arg in ('y', 'yes', 'true'):
-                    self._prefer_os_open = True
-                    self._os_open_modes_allowed = list(range(0, self.DEFAULT_MAX_MODE + 1))
-                    _update_display_msg(suffix=f' (mode <= {oct(self.DEFAULT_MAX_MODE)})')
-                elif arg in ('n', 'no', 'false'):
-                    self._prefer_os_open = False
-                    self._os_open_modes_allowed.clear()
-                else:
-                    raise ValueError(f'Invalid value for `os_open_mode`: {arg}!') from error
-        else:
-            raise ValueError(f'Invalid value for `os_open_mode`: {arg}!')
+    def set_os_mkfifo_allowed_modes(self, value):
+        """
+        Set the allowed modes for `os.mkfifo`.
+
+        Note:
+            This option has no effect on non-UNIX platforms.
+
+        Args:
+            value (str): Option value
+        """
+        self._set_mode_option('os_mkfifo_mode', 'mkfifo', value)
+
+    def set_os_mknod_allowed_modes(self, value):
+        """
+        Set the allowed modes for `os.mknod`.
+
+        Note:
+            This option has no effect on non-UNIX platforms.
+
+        Args:
+            value (str): Option value
+        """
+        self._set_mode_option('os_mknod_mode', 'mknod', value)
 
 
 def register(linter):  # pragma: no cover
@@ -499,5 +718,7 @@ def load_configuration(linter):  # pragma: no cover
     """Load data from the configuration file."""
     for checker in linter.get_checkers():
         if isinstance(checker, SecureCodingStandardChecker):
-            checker.set_os_open_mode(checker.config.os_open_mode)
-            break
+            checker.set_os_open_allowed_modes(checker.config.os_open_mode)
+            checker.set_os_mkdir_allowed_modes(checker.config.os_mkdir_mode)
+            checker.set_os_mkfifo_allowed_modes(checker.config.os_mkfifo_mode)
+            checker.set_os_mknod_allowed_modes(checker.config.os_mknod_mode)
